import os
try:
    import unittest2 as unittest
except:
    import unittest
from rock.config import Config


ROOT_PATH = os.path.dirname(os.path.dirname(__file__))
TESTS_PATH = os.path.join(ROOT_PATH, 'tests')
ASSETS_PATH = os.path.join(TESTS_PATH, 'assets')
CONFIG_PATH = os.path.join(ASSETS_PATH, 'config')
ENV_PATH = os.path.join(ASSETS_PATH, 'env')
DATA_PATH = os.path.join(ASSETS_PATH, 'data')
PROJECT_PATH = os.path.join(ASSETS_PATH, 'project')
USER_PATH = os.path.join(ASSETS_PATH, 'user')


def setenv(mount='test', data='test', user='test'):
    @staticmethod
    def mount_path(*args):
        return os.path.join(*(ENV_PATH, mount) + args)

    @staticmethod
    def data_path(*args):
        return os.path.join(*(DATA_PATH, data) + args)

    @staticmethod
    def user_path(*args):
        return os.path.join(*(USER_PATH, data) + args)

    Config.mount_path = mount_path
    Config.data_path = data_path
    Config.user_path = user_path


class Args(object):

    def __init__(self, **kwargs):
        self.path = os.path.join(PROJECT_PATH, kwargs.get('name', 'simple'))
        self.verbose = True
        self.dry_run = True
        self.runtime = 'test123'
        self.env = 'local'
        for name, value in kwargs.items():
            setattr(self, name, value)


def project(args=None):
    args = args or Args()
    config = {'path': args.path}
    if args.verbose:
        config['verbose'] = True
    if args.dry_run:
        config['dry_run'] = True
        config['verbose'] = True
    if args.runtime:
        config['runtime'] = args.runtime
    from rock.project import Project
    return Project(config, env=args.env)



import argparse
import os
import sys
from rock import __version__
from rock.exceptions import Error
from rock.project import Project
from rock.runtime import list as runtime_list
from rock.text import *


def argument_parser(*args, **kwargs):
    format_usage = kwargs.pop('format_usage', None)
    format_help = kwargs.pop('format_help', None)

    class ArgumentParser(argparse.ArgumentParser):

        def format_usage(self):
            return '%s\n' % format_usage

        def format_help(self):
            return '%s\n%s\n' % (self.format_usage(), format_help)

    kwargs['add_help'] = False

    p = ArgumentParser(*args, **kwargs)
    p.add_argument('--help', action='help')

    return p


def project(args):
    """
    Create and return project instance using cli arguments.
    """
    config = {'path': args.path}
    if args.verbose:
        config['verbose'] = True
    if args.dry_run:
        config['dry_run'] = True
        config['verbose'] = True
    if args.runtime:
        config['runtime'] = args.runtime
    return Project(config, env=args.env)


def config(args, argv):
    """
    Output project configuration (json, yaml).
    """
    parser = argument_parser(prog='rock config', format_usage=CONFIG_USAGE,
                             format_help=CONFIG_HELP)
    parser.add_argument('--format', choices=['json', 'yaml'], default='yaml')

    sub_args = parser.parse_args(argv)

    config = project(args).config
    config.setup()

    if sub_args.format == 'json':
        import json
        sys.stdout.write(json.dumps(config.data, indent=2))
    else:
        import yaml
        sys.stdout.write(yaml.dump(config.data))


def env(args, argv):
    """
    Output project environment.
    """
    parser = argument_parser(prog='rock env', format_usage=ENV_USAGE,
                             format_help=ENV_HELP)

    sub_args = parser.parse_args(argv)

    for name, value in project(args).config['env'].items():
        sys.stdout.write('export %s="%s"\n' % (name, value))


def runtime(args, argv):
    """
    List runtimes install on system.
    """
    parser = argument_parser(prog='rock runtime', format_usage=RUNTIME_USAGE,
                             format_help=RUNTIME_HELP)

    sub_args = parser.parse_args(argv)

    for r in runtime_list():
        sys.stdout.write('%s\n' % r.name)


def main(argv=None):
    """
    Handle command line arguments.
    """

    if argv is None:
        argv = sys.argv[1:]

    # find command position
    pos, skip_next = 0, False
    for i, arg in enumerate(argv):
        if skip_next:
            skip_next = False
            continue
        elif arg.startswith('--'):
            if arg[2:] in ('env', 'path', 'runtime'):
                skip_next = True
        else:
            pos = i
            break

    parser = argument_parser(prog='rock', format_usage=USAGE, format_help=HELP)
    parser.add_argument('--verbose', action='store_true')
    parser.add_argument('--dry-run', action='store_true')
    parser.add_argument('--version', action='version', version=__version__)
    parser.add_argument('--env', default=os.environ.get('ROCK_ENV', 'local'))
    parser.add_argument('--path', default=os.environ.get('ROCK_PATH', ''))
    parser.add_argument('--runtime', default=os.environ.get('ROCK_RUNTIME'))
    parser.add_argument('command')

    try:
        # only parse up until command
        args = parser.parse_args(argv[:pos + 1])
        if args.command in ('config', 'env', 'runtime'):
            globals()[args.command](args, argv[pos + 1:])
        else:
            project(args).run(args.command, argv[pos + 1:])
    except Error, error:
        message = '%s' % error
        if not message.endswith('\n'):
            message += '\n'
        parser.exit(1, message)



import helper
from rock import runtime


class RuntimeTestCase(helper.unittest.TestCase):

    def setUp(self):
        helper.setenv()

    def test_list(self):
        rs = runtime.list()
        self.assertEqual(rs[0].name, 'parent123')
        self.assertEqual(rs[1].name, 'parse123')
        self.assertEqual(rs[2].name, 'test123')
        self.assertEqual(rs[3].name, 'user123')
        @staticmethod
        def not_found():
            return '/tmp/this-should-not-exist'
        runtime.Runtime.root_path = not_found
        runtime.Runtime.user_path = not_found
        rs = runtime.list()
        self.assertEqual(len(rs), 0)
        self.assertTrue(isinstance(rs, list))



import helper
import os
import shutil
import tempfile
from rock.config import Config
from rock.exceptions import ConfigError


class ConfigTestCase(helper.unittest.TestCase):

    def setup(self, name='simple', data='test', config=None, env='local'):
        helper.setenv(data=data)
        self.path = os.path.join(helper.PROJECT_PATH, name)
        self.env = env
        if config is None:
            config = {'path': self.path}
        return Config(config, env=env)

    def full(self, c):
        self.assertEqual(c['path'], self.path)
        self.assertEqual(c['runtime'], 'test123')
        # env
        env = c['env']
        self.assertEqual(env['PATH'],
            '%s/bindir:/opt/rock/runtime/test123/usr/bin:${PATH}' %
            self.path)
        self.assertEqual(env['BUILD_PATH'], 'build_path')
        self.assertEqual(env['TEST_PATH'], 'test_path')
        self.assertEqual(env['ROCK_PATH'], self.path)
        # build with parent
        build = [b for b in c['build'].split('\n') if b]
        self.assertEqual(len(build), 3)
        self.assertEqual(build[0], 'pre')
        self.assertEqual(build[1], 'build')
        self.assertEqual(build[2], 'post')
        # other tasks
        self.assertEqual(c['build_deployment'].strip(), 'build deployment')
        self.assertEqual(c['clean'].strip(), 'clean')
        self.assertEqual(c['test'].strip(), 'test')
        # misc
        self.assertTrue('build' in c)
        self.assertTrue('build' in iter(c))

    def test_paths(self):
        paths = Config.paths('ok')
        self.assertEqual(len(paths), 3)
        self.assertTrue(paths[0].endswith('/user/test/ok'))
        self.assertTrue(paths[1].endswith('/test/etc/rock/ok'))
        self.assertTrue(paths[2].endswith('/data/test/ok'))

    def test_parent(self):
        c = self.setup('parent')
        self.assertEqual(c['test_parent1'], '1')
        self.assertEqual(c['test_parent2'], '2')
        # str to dict
        self.assertEqual(c['test_str_to_str'], 'pre-simple1-post')
        self.assertEqual(c['test_str_to_dict']['one'], 'pre1-simple2-post1')
        self.assertEqual(c['test_str_to_dict']['two'], 'pre2-simple2-post2')
        # dict to dict
        self.assertEqual(c['test_dict_to_dict']['one'], 'pre1-simple1-post1')
        self.assertEqual(c['test_dict_to_dict']['two'], 'pre2--post2')
        self.assertEqual(c['test_dict_to_dict']['three'], 'simple3')
        self.assertEqual(c['test_dict_to_dict']['four'], 'pre4-simple4-post4')

    def test_parent2(self):
        c = self.setup('parent2')
        with self.assertRaisesRegexp(ConfigError, r'^unable to merge') as a:
            c['test_dict_to_dict']

    def test_simple1(self):
        c = self.setup()
        self.full(c)
        self.assertEqual(c['env']['HELLO'], 'world')
        # misc other
        self.assertEqual(len(c), 10)

    def test_simple2(self):
        c = self.setup(data='test123', env='prod')
        self.full(c)
        self.assertEqual(c['env']['HELLO'], 'better world')

    def test_path(self):
        rock_path = os.path.join(helper.PROJECT_PATH, 'path')
        cwd = os.getcwd()
        try:
            os.chdir(os.path.join(rock_path, 'tests'))
            c = self.setup('path', config={})
            env = c['env']
            self.assertEqual(env['ROCK_PATH'], rock_path)
        finally:
            os.chdir(cwd)

    def test_no_valid_config(self):
        rock_path = tempfile.mkdtemp()
        cwd = os.getcwd()
        try:
            os.chdir(rock_path)
            c = self.setup('path', config={})
            with self.assertRaisesRegexp(ConfigError, r"runtime is required") as a:
                c['path']
        finally:
            os.chdir(cwd)
            shutil.rmtree(rock_path)

    def test_runtime_notfound(self):
        c = self.setup('runtime_notfound')
        with self.assertRaisesRegexp(ConfigError, r"runtime path doesn't exist") as a:
            c['path']

    def test_runtime_config_notfound(self):
        c = self.setup('runtime_config_notfound')
        with self.assertRaisesRegexp(ConfigError, r'not found: .*') as a:
            c['path']

    def test_project_parse(self):
        c = self.setup('project_parse')
        with self.assertRaisesRegexp(ConfigError, r'.rock.yml syntax error') as a:
            c['path']

    def test_parse(self):
        c = self.setup('parse')
        with self.assertRaisesRegexp(ConfigError, r'parse error: .*') as a:
            c['path']

    def test_badenv1(self):
        c = self.setup('badenv1')
        with self.assertRaisesRegexp(ConfigError, r'env must be an associative array') as a:
            c['path']

    def test_badenv2(self):
        c = self.setup('badenv2')
        with self.assertRaisesRegexp(ConfigError, r'env.one must be a string') as a:
            c['path']



import helper
from rock import utils
from rock.exceptions import ConfigError


class UtilsTestCase(helper.unittest.TestCase):

    def test_shell(self):
        utils.Shell.run = lambda self: self
        s = utils.Shell()
        self.assertTrue(isinstance(s.__enter__(), utils.Shell))
        s.write('ok')
        s.__exit__(None, None, None)
        self.assertEqual(s.stdin.getvalue(), 'ok\n')
        def execl(*args):
            self.assertEqual(len(args), 4)
            self.assertEqual(args[0], '/bin/bash')
            self.assertEqual(args[1], '-l')
            self.assertEqual(args[2], '-c')
            self.assertEqual(args[3], 'ok\n')
        utils.os.execl = execl
        s.__exit__('type', 'value', 'tracebook')

    def test_noshell(self):
        utils.ROCK_SHELL = '/tmp/hopefully-no-exists'
        s = utils.Shell()
        s.__enter__()
        self.assertRaises(ConfigError, s.__exit__, 'type', 'value', 'tracebook')



def _(text):
    return text.strip('\n')

USAGE = _("""
Usage: rock [--help] [--env=ENV] [--path=PATH] [--runtime=RUNTIME] command
""")

HELP = _("""
  --help             show help message
  --verbose          show script while running
  --dry-run          show script without running
  --version          show version

project:
  --env=ENV          set env
  --path=PATH        set path
  --runtime=RUNTIME  set runtime

commands:
  build              run build
  test               run tests
  run                run in environment
  clean              clean project files

other commands:
  config             show project configuration
  env                show evaluable environment variables
  init               generates project skeleton
  runtime            show installed runtimes
""")

CONFIG_USAGE = _("""
Usage: rock config [--format=FORMAT]
""")

CONFIG_HELP = _("""
  --help             show help message
  --format           set output format (json, yaml)
""")

ENV_USAGE = _("""
Usage: rock env
""")

ENV_HELP = _("""
  --help             show help message
""")

RUNTIME_USAGE = _("""
Usage: rock runtime
""")

RUNTIME_HELP = _("""
  --help             show help message
""")



import helper
import os
from rock.config import Config

class BeforeTestCase(helper.unittest.TestCase):

    def test_config(self):
        c = Config({'path': ''}, env='local')
        self.assertEqual(c.data_path(), os.path.realpath(os.path.join(
            os.path.dirname(__file__), '..', 'rock', 'data')))
        self.assertEqual(c.mount_path(), '/')
        self.assertEqual(c.user_path(), os.path.expanduser('~/.rock'))



import collections
import copy
import os
import re
import string
import yaml
from rock.exceptions import ConfigError


PARENT_RE = re.compile(r'\{\{\s*parent\s*\}\}', re.MULTILINE)


class Config(collections.Mapping):
    """
    Parse and merge configuration files.
    """

    def __init__(self, data, env=None):
        self.data = data
        self.env = env
        self._setup = False

    @staticmethod
    def paths(*args):
        return (
            Config.user_path(*args),
            Config.etc_path(*args),
            Config.data_path(*args),
        )

    @staticmethod
    def data_path(*args):
        return os.path.join(*(os.path.dirname(__file__), 'data') + args)

    @staticmethod
    def etc_path(*args):
        return Config.mount_path(*('etc', 'rock') + args)

    @staticmethod
    def mount_path(*args):
        return os.path.join(*('/',) + args)

    @staticmethod
    def user_path(*args):
        return os.path.join(*(os.path.expanduser('~/.rock'),) + args)

    @staticmethod
    def runtime(*args, **kwargs):
        from rock.runtime import Runtime
        return Runtime(*args, **kwargs)

    @staticmethod
    def parse(path, require_exists=True, require_parses=True):
        """
        Parse and return configuration file.
        """
        if not os.path.isfile(path):
            if require_exists:
                raise ConfigError('not found: ' + path)
            else:
                return None
        try:
            with open(path) as f:
                return yaml.safe_load(f)
        except Exception, error:
            if require_parses:
                raise ConfigError('parse error: ' + path)

    def merge_env(self, src, dst, env=None):
        """
        Merge environment variables.
        """
        env = 'env_%s' % env if env else 'env'
        if env in src:
            if not isinstance(src[env], dict):
                raise ConfigError('%s must be an associative array' % env)
            # evaluate env variables
            for name, value in src[env].items():
                if not isinstance(value, basestring):
                    if isinstance(value, (int, float)):
                        src[env][name] = str(value)
                    else:
                        raise ConfigError('%s.%s must be a string' %
                                          (env, name))
                dst['env'][name] = string.Template(
                    src[env][name]).safe_substitute(**dst['env'])
            del src[env]

    def merge(self, src, dst):
        if src is None:
            return dst
        # merge global environment variables
        self.merge_env(src, dst)
        # merge env-specific environment variables
        self.merge_env(src, dst, self.env)
        # merge sections
        for name in src.keys():
            if name not in dst:
                if isinstance(src[name], basestring):
                    src[name] = PARENT_RE.sub('', src[name])
                elif isinstance(src[name], dict):
                    for subname in src[name]:
                        value = src[name][subname]
                        if isinstance(value, basestring):
                            src[name][subname] = PARENT_RE.sub('', value)
                dst[name] = src[name]
            elif isinstance(src[name], basestring):
                if not isinstance(dst[name], basestring):
                    raise ConfigError('unable to merge "%s" into "str"' %
                                      type(dst[name]).__name__)
                dst[name] = PARENT_RE.sub(dst[name], src[name])
            elif isinstance(src[name], dict):
                dst_is_dict = isinstance(dst[name], dict)
                for subname in src[name]:
                    if isinstance(dst[name], basestring):
                        src[name][subname] = PARENT_RE.sub(dst[name],
                                                           src[name][subname])
                    elif dst_is_dict:
                        if subname in dst[name]:
                            src[name][subname] = PARENT_RE.sub(
                                dst[name][subname],
                                src[name][subname],
                            )
                            del dst[name][subname]
                        else:
                            src[name][subname] = PARENT_RE.sub(
                                '',
                                src[name][subname],
                            )
                if dst_is_dict:
                    src[name].update(dst[name])
                dst[name] = src[name]
            del src[name]
        dst.update(src)

    def setup_path(self):
        if not self.data.get('path'):
            path = '.'
            while os.path.split(os.path.abspath(path))[1]:
                config_path = os.path.join(path, '.rock.yml')
                if os.path.isfile(config_path):
                    self.data['path'] = os.path.abspath(path)
                    break
                path = os.path.join('..', path)
        if not self.data.get('path'):
            self.data['path'] = os.getcwd()

    def setup(self):
        if self._setup:
            return
        self._setup = True
        # path
        self.setup_path()
        # new configuration
        data = {}
        # runtime
        yml_path = os.path.join(self.data['path'], '.rock.yml')
        if yml_path and os.path.isfile(yml_path):
            data = self.parse(yml_path)
            if not isinstance(data, dict):
                raise ConfigError('.rock.yml syntax error')
            data.update(self.data)
        else:
            data = copy.deepcopy(self.data)
        # project
        for name in ('path', 'runtime'):
            if name not in data:
                raise ConfigError('%s is required' % name)
        # paths
        runtime = self.runtime(data['runtime'])
        etc_path = self.etc_path('runtime')
        runtime_yml = data['runtime'] + '.yml'
        data_path = self.data_path('runtime', runtime_yml)
        if not os.path.exists(data_path):
            file_name = data['runtime'].rstrip('0123456789') + '.yml'
            data_path = self.data_path('runtime', file_name)
        # ensure runtime exists
        if not os.path.isdir(runtime.path()):
            raise ConfigError("runtime path doesn't exist")
        # parse configs
        runtime_config = self.parse(runtime.path('rock.yml'))
        rock_config = self.parse(data_path, require_exists=False)
        etc_config = self.parse(os.path.join(etc_path, runtime_yml),
                                require_exists=False)
        # merge
        self.data = {
            'env': {
                'ROCK_ENV': self.env,
                'ROCK_PATH': data['path'],
                'ROCK_RUNTIME': data['runtime'],
            },
        }
        # merge runtime
        self.merge(runtime_config, self.data)
        # merge runtime config
        self.merge(rock_config, self.data)
        self.merge(etc_config, self.data)
        # merge project
        self.merge(data, self.data)

    def __contains__(self, *args, **kwargs):
        self.setup()
        return self.data.__contains__(*args, **kwargs)

    def __getitem__(self, *args, **kwargs):
        self.setup()
        return self.data.__getitem__(*args, **kwargs)

    def __iter__(self, *args, **kwargs):
        self.setup()
        return self.data.__iter__(*args, **kwargs)

    def __len__(self, *args, **kwargs):
        self.setup()
        return self.data.__len__(*args, **kwargs)



import os
import pipes
import re
from rock.config import Config
from rock.exceptions import ConfigError
from rock.utils import Shell

NAME_RE = re.compile('^[a-zA-Z0-9_]+$')


class Project(object):

    def __init__(self, *args, **kwargs):
        self.config = Config(*args, **kwargs)

    def run(self, section, argv=None):
        argv, args, opts, opts_list = argv or [], [], {}, []
        script = self.config.get(section, '')

        # ensure section exists
        def check():
            if section not in self.config:
                raise ConfigError('section not found: %s' % section)
            if not isinstance(self.config[section], basestring):
                raise ConfigError('section must be a string: %s' % section)

        # handle run special case
        if section == 'run':
            if not argv:
                check()
            else:
                script = ' '.join(map(pipes.quote, argv))
        else:
            check()
        # build bash script
        with Shell() as shell:
            # print commands as they're run
            if self.config.get('verbose'):
                shell.write('set -o verbose')
            # declare builtin functions
            shell.write('warn() { echo "$@" >&2; }')
            shell.write('die() { warn "$@"; exit 1; }')
            # don't execute commands, just print them
            if self.config.get('dry_run'):
                shell.write('set -o noexec')
            # exit with error if any one command fails
            shell.write('set -o errexit')
            # switch to project directory
            if section != 'run' or len(argv) == 0:
                shell.write('cd ' + pipes.quote(self.config['path']))
            # setup environment variables
            if self.config.get('env'):
                # blank line before exports
                for name, value in self.config['env'].items():
                    shell.write('export %s="%s"' % (name, value))
            # handle arguments
            if section != 'run':
                # raw arguments
                shell.write("export ROCK_ARGV='%s'" %
                            ' '.join(map(pipes.quote, argv)))
                # parse arguments
                for i, arg in enumerate(argv):
                    if arg.startswith('--'):
                        name, value = arg[2:], 'true'
                        if '=' in name:
                            name, value = name.split('=', 1)
                        name = name.upper().replace('-', '_')
                        if NAME_RE.match(name):
                            opts[name] = value
                            if name in opts_list:
                                opts_list.remove(name)
                            opts_list.append(name)
                    else:
                        args.append(arg)
                # parsed arguments
                shell.write("export ROCK_ARGS='%s'" %
                            ' '.join(map(pipes.quote, args)))
                # set zero argument to command
                args.insert(0, section)
                # positional arguments
                for i, arg in enumerate(args):
                    shell.write("export ROCK_ARG%s='%s'" %
                                (i, pipes.quote(arg)))
                # parsed argument options
                for name, value in opts.items():
                    shell.write('export ROCK_ARGS_%s="%s"' %
                                (name, pipes.quote(value)))
                # parsed options
                shell.write("export ROCK_OPTS='%s'" %
                            ' '.join(map(pipes.quote, opts_list)))
                shell.write("export ROCK_CWD='%s'" % os.getcwd())
            # execute script
            shell.write('# script')
            shell.write(script)



import helper
import os
import subprocess
import sys
import tempfile
from StringIO import StringIO
from rock import cli, utils
from rock.exceptions import ConfigError
from rock.project import Project
from helper import Args


def config_file(name):
    with open(os.path.join(helper.CONFIG_PATH, name)) as f:
        return f.read().strip()


class CliTestCase(helper.unittest.TestCase):

    def setUp(self):
        helper.setenv()
        def execl(*args):
            self.args = args
        utils.os.execl = execl
        self.old_stdout = sys.stdout
        self.old_stderr = sys.stderr
        self.stdout = sys.stdout = StringIO()
        self.stderr = sys.stderr = StringIO()

    def setTeardown(self):
        sys.stdout = self.old_stdout
        sys.stderr = self.old_stderr

    def test_project(self):
        self.assertTrue(isinstance(cli.project(Args()), Project))

    def test_config_json(self):
        cli.config(Args(), ['--format=json'])
        data = self.stdout.getvalue().strip().replace(helper.ROOT_PATH, '<ROOT>')
        self.assertEqual(data, config_file('data.json'))

    def test_config_yaml(self):
        cli.config(Args(), ['--format=yaml'])
        data = self.stdout.getvalue().strip().replace(helper.ROOT_PATH, '<ROOT>')
        self.assertEqual(data, config_file('data.yaml'))

    def test_env(self):
        cli.env(Args(), [])
        self.assertTrue('\nexport TEST_PATH="test_path"\n' in self.stdout.getvalue())

    def test_runtime(self):
        cli.runtime(Args(), [])
        self.assertTrue('\ntest123\n' in self.stdout.getvalue())

    def test_help(self):
        self.assertRaises(SystemExit, cli.main, ['--help'])
        self.assertEqual(self.stdout.getvalue(), cli.USAGE + '\n\n' + cli.HELP + '\n')

    def test_main_empty(self):
        stderr = sys.stderr
        argv = sys.argv
        try:
            sys.argv = []
            sys.stderr = StringIO()
            self.assertRaises(SystemExit, cli.main)
            self.assertTrue('Usage: rock' in sys.stderr.getvalue())
        finally:
            sys.argv = argv
            sys.stderr = stderr

    def test_main_valid(self):
        cli.main(argv=['--env', 'prod', 'runtime'])

    def test_main_invalid(self):
        stderr = sys.stderr
        try:
            sys.stderr = StringIO()
            self.assertRaises(SystemExit, cli.main, ['--path=%s/not-found' % helper.TESTS_PATH, 'run', 'ok'])
        finally:
            sys.stderr = stderr



import helper
from rock import utils
from rock.exceptions import ConfigError


class ProjectTestCase(helper.unittest.TestCase):

    def setUp(self):
        helper.setenv()
        def execl(*args):
            self.args = args
        utils.os.execl = execl
        self.p = helper.project()

    @property
    def split(self):
        return self.args[3].split('\n# script\n', 1)

    @property
    def script(self):
        return self.split[1].strip()

    def test_arg(self):
        self.p.run('clean', ['--six', '--zero', 'one', '--two=2', 'three', 'four', '--five=five', '--six'])
        data = {}
        for value in self.split[0].split('\n'):
            if not value.startswith('export '):
                continue
            name = value.split(' ', 1)[1]
            if '=' not in name:
                continue
            data[name.split('=', 1)[0]] = value
        self.assertEqual(data.get('ROCK_ENV'), 'export ROCK_ENV="local"')
        self.assertEqual(data.get('ROCK_ARGV'), "export ROCK_ARGV='--six --zero one --two=2 three four --five=five --six'")
        self.assertEqual(data.get('ROCK_ARGS'), "export ROCK_ARGS='one three four'")
        self.assertEqual(data.get('ROCK_ARG0'), "export ROCK_ARG0='clean'")
        self.assertEqual(data.get('ROCK_OPTS'), "export ROCK_OPTS='ZERO TWO FIVE SIX'")

    def test_run(self):
        self.p.run('run')
        self.assertEqual(self.script, 'echo zero')

    def test_run_manual(self):
        self.p.run('run', ['echo', 'hello'])
        self.assertEqual(self.script, 'echo hello')

    def test_run_no_section(self):
        self.p = helper.project(helper.Args(name='parent'))
        self.assertRaises(ConfigError, self.p.run, 'run')

    def test_run_section_not_string(self):
        self.p = helper.project(helper.Args(name='parent'))
        self.assertRaises(ConfigError, self.p.run, 'test_str_to_dict')

    def test_build(self):
        self.p.run('build')
        self.assertEqual(self.script, 'pre\n\nbuild\n\npost')

    def test_clean(self):
        self.p.run('clean')
        self.assertEqual(self.script, 'clean')



import sys

from distutils.core import setup
from rock import __version__ as version

long_description = """
rock is a command line tool that simplifies building, testing, and running
applications in multiple languages and versions on the same system.

See http://www.rockstack.org for more information.
"""

install_requires = ['PyYAML']

if sys.version_info < (2, 7):
    install_requires += ['argparse']

setup(
    name='rock',
    version=version,
    description='Rock Better Runtimes',
    long_description=long_description.strip(),
    author='Silas Sewell',
    author_email='silas@sewell.org',
    license='MIT',
    url='http://www.rockstack.org',
    packages=['rock'],
    package_data={'rock': ['data/*/*']},
    scripts=['scripts/rock'],
    install_requires=install_requires,
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
)



import StringIO
import os
from rock.exceptions import ConfigError


ROCK_SHELL = (os.environ.get('ROCK_SHELL') or '/bin/bash -c').split()
ROCK_SHELL.insert(1, os.path.basename(ROCK_SHELL[0]))


def isexecutable(path):
    return os.path.isfile(path) and os.access(path, os.X_OK)


class Shell(object):

    def __init__(self):
        self.stdin = StringIO.StringIO()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.run()

    def run(self):
        if not isexecutable(ROCK_SHELL[0]):
            raise ConfigError('invalid ROCK_SHELL: %s' % ROCK_SHELL)
        os.execl(*(ROCK_SHELL + [self.stdin.getvalue()]))

    def write(self, text):
        self.stdin.write(text + '\n')
